% subroutine of MAIN.m to solve for aggregate law of motion

function [kmts,kcross]  = AGGREGATE(T,idshock,agshock,km_max,km_min,kprime,km,k,epsilon2,k_min,k_max,kcross,a2)

kmts=zeros(T,1);         % a time series of the mean of capital distribution 

for t=1:T
  
   kmts(t)=mean(kcross); 
   % define the period t moment of kmts as
   % the mean of the t-th period cross-sectional 
   % distribution of capital (only first moment)
   
   kmts(t)=kmts(t)*(kmts(t)>=km_min)*(kmts(t)<=km_max)+km_min*(kmts(t)<km_min)+km_max*(kmts(t)>km_max); 
   % restrict kmts to be within [km_min, km_max]
   % To find kmts(t+1), we should compute a new cross-sectional distribution 
   % at t+1. For this purpose, we first find kprime by interpolation for 
   % realized kmts(t) and agshock(t) (kprimet below) and then use it to 
   % compute new kcross by interpolation (kcrossn below) given the previous 
   % kcross and the realized idshock(t)
   
   kprimet4=interpn(k,km,a2,epsilon2,kprime,k, kmts(t),agshock(t),epsilon2,'spline');
   % sample points: k,km,a2,epsilon2;
   % value: kprime;
   % query points: k, kmts(t),agshock(t),epsilon2;
      % a 4-dimensional capital function at time t is obtained by fixing
      % known kmts(t) and agshock (t)
      
   kprimet=squeeze(kprimet4); 
   % the size of kprimet4 is ngridk*1*1*nstates_id; 
   % in kprimet, all singleton dimensions 
   % (i.e., those with only one column per page) are removed;
   % now the dimension is ngridk*nstates_id;
                              
   kcrossn=interpn(k,epsilon2,kprimet,kcross,idshock(t,:),'spline'); 
   % sample points: k,epsilon2;
   % value: kprimet;
   % query points: kcross,idshock(t,:);
     % given kcross and idiosyncratic shocks we compute kcrossn
                              
   kcrossn=kcrossn.*(kcrossn>=k_min).*(kcrossn<=k_max)+k_min*(kcrossn<k_min)+k_max*(kcrossn>k_max); 
   % restrict kcrossn to be within [k_min, k_max]
                              
   kcross=kcrossn;
   
end
