% subroutine of MAIN.m to solve for individual policy function 

function [kprime,c]  = INDIVIDUAL(prob,ur_b,ur_g,ngridk,ngridkm,nstates_ag,nstates_id,k,km,er_b,er_g,a,epsilon,l_bar,alpha,delta,gamma,beta,mu,km_max,km_min,kprime,B,criter_k,k_min,k_max,update_k);
% function [kprime,c] = INDIVIDUAL(prob,...) declares a function named INDIVIDUAL 
% that accepts inputs (prob,...) and UPDATE kprime and c,
% where kprime is capital next period or saving today; and c is consumption today;
% Initial kprime is guess as 0.9k in step I out of III in individual problem
%__________________________________________________________________________
%
% Auxilary matrices of transition probabilities (needed for computing the 
% expectation term in the Euler equation)  
%__________________________________________________________________________

% grid of points for (k, m, a, epsilon), the sizes of each dimension are
% [ngridk, ngridkm, nstates_ag and nstates_id]:

prob_bu=zeros(ngridk,ngridkm,nstates_ag,nstates_id); 
% for a bad agg. state and unemployed idios. state in the next period
prob_be=zeros(ngridk,ngridkm,nstates_ag,nstates_id); 
% for a bad agg. state and employed idios. state in the next period
prob_gu=zeros(ngridk,ngridkm,nstates_ag,nstates_id); 
% for a good agg. state and unemployed idios. state in the next period
prob_ge=zeros(ngridk,ngridkm,nstates_ag,nstates_id); 
% for a good agg. state and employed idios. state in the next period

% The prob is relevant to agg. and idios. states only.
% Thus we only need to control the last two.

prob_bu(:,:,1,1)=prob(1,1)*ones(ngridk,ngridkm);
% prob(1,1) is prob from (b,u) to (b,u)
prob_bu(:,:,1,2)=prob(2,1)*ones(ngridk,ngridkm);
% prob(2,1) is prob from (b,e) to (b,u)
prob_bu(:,:,2,1)=prob(3,1)*ones(ngridk,ngridkm);
% prob(3,1) is prob from (g,u) to (b,u)
prob_bu(:,:,2,2)=prob(4,1)*ones(ngridk,ngridkm);
% prob(4,1) is prob from (g,e) to (b,u)

prob_be(:,:,1,1)=prob(1,2)*ones(ngridk,ngridkm);
% prob(1,2) is prob from (b,u) to (b,e)
prob_be(:,:,1,2)=prob(2,2)*ones(ngridk,ngridkm);
% prob(2,2) is prob from (b,e) to (b,e)
prob_be(:,:,2,1)=prob(3,2)*ones(ngridk,ngridkm);
% prob(3,2) is prob from (g,u) to (b,e)
prob_be(:,:,2,2)=prob(4,2)*ones(ngridk,ngridkm);
% prob(4,2) is prob from (g,e) to (b,e)

prob_gu(:,:,1,1)=prob(1,3)*ones(ngridk,ngridkm);
% prob(1,3) is prob from (b,u) to (g,u)
prob_gu(:,:,1,2)=prob(2,3)*ones(ngridk,ngridkm);
% prob(2,3) is prob from (b,e) to (g,u)
prob_gu(:,:,2,1)=prob(3,3)*ones(ngridk,ngridkm);
% prob(3,3) is prob from (g,u) to (g,u)
prob_gu(:,:,2,2)=prob(4,3)*ones(ngridk,ngridkm);
% prob(4,3) is prob from (g,e) to (g,u)

prob_ge(:,:,1,1)=prob(1,4)*ones(ngridk,ngridkm);
% prob(1,4) is prob from (b,u) to (g,e)
prob_ge(:,:,1,2)=prob(2,4)*ones(ngridk,ngridkm);
% prob(2,4) is prob from (b,e) to (g,e)
prob_ge(:,:,2,1)=prob(3,4)*ones(ngridk,ngridkm);
% prob(3,4) is prob from (g,u) to (g,e)
prob_ge(:,:,2,2)=prob(4,4)*ones(ngridk,ngridkm);
% prob(4,4) is prob from (g,e) to (g,e)

%__________________________________________________________________________
% 
% Auxilary matrices (needed for computing interest rate, wage and wealth)  
%__________________________________________________________________________

% Individual capital
kaux=zeros(ngridk,ngridkm,nstates_ag,nstates_id);
        % k is a ngridk*1 matrix, k*ones(1,ngridkm) is ngridk*ngridkm matrix.
kaux(:,:,1,1)=k*ones(1,ngridkm);
% k-grid when (b,u)
kaux(:,:,1,2)=k*ones(1,ngridkm);
% k-grid when (b,e)
kaux(:,:,2,1)=k*ones(1,ngridkm);
% k-grid when (g,u)
kaux(:,:,2,2)=k*ones(1,ngridkm);
% k-grid when (g,e)

% First moments (mean) of capital distribution (km)
kmaux=zeros(ngridk,ngridkm,nstates_ag,nstates_id);
        % km is ngridkm*1, ones(ngridk,1)*km' is ngridk*ngridkm matrix.
kmaux(:,:,1,1)=ones(ngridk,1)*km';
% k mean grid when (b,u)
kmaux(:,:,1,2)=ones(ngridk,1)*km';
% k mean grid when (b,e)
kmaux(:,:,2,1)=ones(ngridk,1)*km';
% k mean grid when (g,u)
kmaux(:,:,2,2)=ones(ngridk,1)*km';
% k mean grid when (g,e)

% Aggregate labor (denoted by L in the paper),
aglabor=zeros(ngridk,ngridkm,nstates_ag,nstates_id);
        % with employment rate (er) only depend on agg. states.
aglabor(:,:,1,:)=er_b*ones(ngridk,ngridkm,nstates_id);
% er_b: employment rate when bad (=1)
aglabor(:,:,2,:)=er_g*ones(ngridk,ngridkm,nstates_ag);
% er_b: employment rate when good (=2)

% Aggregate shock (a)
agshock_aux=zeros(ngridk,ngridkm,nstates_ag,nstates_id);
agshock_aux(:,:,1,:)=a(1)*ones(ngridk,ngridkm,nstates_id);
% a(1): aggregate state when bad (=1)
agshock_aux(:,:,2,:)=a(2)*ones(ngridk,ngridkm,nstates_id);
% a(2): aggregate state when good (=2)

% Idiosyncratic shock (epsilon)
idshock_aux=zeros(ngridk,ngridkm,nstates_ag,nstates_id);
idshock_aux(:,:,:,1)=epsilon(1)*ones(ngridk,ngridkm,nstates_ag);
% epsilon(1): individual state when unemployed (=1)
idshock_aux(:,:,:,2)=epsilon(2)*ones(ngridk,ngridkm,nstates_ag);
% epsilon(2): individual state when employed (=2)

%__________________________________________________________________________
%
% Interest rate, wage and wealth under given k and km
%__________________________________________________________________________

ones4=ones(ngridk,ngridkm,nstates_ag,nstates_ag); 
% a four-dimensional matrix of ones

% Interest rate
irateaux=alpha*(agshock_aux.*(kmaux./aglabor/l_bar).^(alpha-1)); 
% a 100*4*2*2 matrix of interest rates
% interest rate=MPK

% Wage
wageaux=(1-alpha)*(agshock_aux.*(kmaux./aglabor/l_bar).^alpha); 
% a 100*4*2*2 dimensional matrix of wages
% wage=MPL

% Wealth
wealth=irateaux.*kaux+(wageaux.*idshock_aux)*l_bar+mu*(wageaux.*(ones4-idshock_aux))+(1-delta)*kaux-mu*(wageaux.*(1-aglabor)./aglabor).*idshock_aux; 
% a 100*4*2*2-dimensional matrix of individual wealth
% Budget constraint
%__________________________________________________________________________
% 
% A New mean of capital distribution (km') for the given vector of the ALM
% coefficients B (denoted by b in the paper)
%__________________________________________________________________________

% Remind that (the ALM for a bad aggregate state is ln(km')=B(1)+B(2)*ln(km) 
% and that for a good aggregate state is ln(km')=B(3)+B(4)*ln(km)
% B matrix is set at initial value now

% Mean of capital stock at next period
kmprime=zeros(ngridk,ngridkm,nstates_ag,nstates_id);

% Aggregate Law of motion (ALM) of km under different states
kmprime(:,:,1,1)=exp(B(1)*ones(ngridk,ngridkm)+B(2)*log(kmaux(:,:,1,1)));
kmprime(:,:,1,2)=exp(B(1)*ones(ngridk,ngridkm)+B(2)*log(kmaux(:,:,1,2)));
kmprime(:,:,2,1)=exp(B(3)*ones(ngridk,ngridkm)+B(4)*log(kmaux(:,:,2,1)));
kmprime(:,:,2,2)=exp(B(3)*ones(ngridk,ngridkm)+B(4)*log(kmaux(:,:,2,2)));
                                        
kmprime=(kmprime>=km_min).*(kmprime<=km_max).*kmprime+(kmprime<km_min)*km_min+(kmprime>km_max)*km_max;
% restricting km' to be in [km_min,km_max] range

%__________________________________________________________________________
%
% Future interest rate and wage conditional on state
%__________________________________________________________________________

irate_b=alpha*a(1)*((kmprime./(er_b*ones4*l_bar)).^(alpha-1));  
% under a bad future aggregate state
irate_g=alpha*a(2)*((kmprime./(er_g*ones4*l_bar)).^(alpha-1));  
% under a good future aggregate state
                                      
wage_b=(1-alpha)*a(1)*((kmprime./(er_b*ones4*l_bar)).^(alpha)); 
% under a bad future aggregate state
wage_g=(1-alpha)*a(2)*((kmprime./(er_g*ones4*l_bar)).^(alpha)); 
% under a good future aggregate state

%__________________________________________________________________________
% 
% SOLVING THE INDIVIDUAL PROBLEM
%         to solve for k''(by interpolating the grid given k and k'), c', u'(c').
%         do so for [four different states: bu, be, gu, ge]
%__________________________________________________________________________   

dif_k=1;
   while dif_k>criter_k
       
 % dif_k: difference between coefficients B of the the ALM on 
 % successive iterations; initially, set to a large number
 % criter_k=1e-8: convergence criterion for the individual capital function
   

% Bad aggregate state and unemployed idiosyncratic state 
     k2prime_bu=interpn(k,km,kprime(:,:,1,1),kprime,kmprime,'spline');
     % sample points: k,km;
     % value: kprime(:,:,1,1);
     % query points: kprime,kmprime; 
     
     % finding the individual policy function k''=k(k',km') by interpolating
     % the previously found policy function k'=k(k,km) in new points (k',km')
 
    
     cprime_bu=irate_b.*kprime+mu*(wage_b.*ones4)+(1-delta)*kprime-k2prime_bu; 
     % future consumption (c') deduced from budget constraint
     
     cprime_bu=(cprime_bu>0).*cprime_bu+(cprime_bu<=0)*10^-10; 
     % if c'<=0, then set it to a very low positive number 
     muprime_bu=cprime_bu.^(-gamma); 
     % marginal utility of future consumption
   
% Bad aggregate state and employed idiosyncratic state
   
     k2prime_be=interpn(k,km,kprime(:,:,1,2),kprime,kmprime,'spline');
     % sample points: k,km;
     % value: kprime(:,:,1,2);
     % query points: kprime,kmprime; 
     
     
     cprime_be=irate_b.*kprime+wage_b.*(epsilon(2)*l_bar*ones4)+(1-delta)*kprime-mu*(wage_b.*((ur_b./(1-ur_b))*ones4))-k2prime_be;
     cprime_be=(cprime_be>0).*cprime_be+(cprime_be<=0)*10^-10;
     muprime_be=cprime_be.^(-gamma);
 
% Good aggregate state and unemployed idiosyncratic state
   
     k2prime_gu=interpn(k,km,kprime(:,:,2,1),kprime,kmprime,'spline');
     % sample points: k,km;
     % value: kprime(:,:,2,1);
     % query points: kprime,kmprime; 
     
     cprime_gu=irate_g.*kprime+mu*(wage_g.*ones4)+(1-delta)*kprime-k2prime_gu;
     cprime_gu=(cprime_gu>0).*cprime_gu+(cprime_gu<=0)*10^-10;
     muprime_gu=cprime_gu.^(-gamma);
   
% Good aggregate state and employed idiosyncratic state
   
     k2prime_ge=interpn(k,km,kprime(:,:,2,2),kprime,kmprime,'spline');
     % sample points: k,km;
     % value: kprime(:,:,2,2);
     % query points: kprime,kmprime; 
     
     cprime_ge=irate_g.*kprime+wage_g.*(epsilon(2)*l_bar*ones4)+(1-delta)*kprime-mu*(wage_g.*((ur_g./(1-ur_g))*ones4))-k2prime_ge;
     cprime_ge=(cprime_ge>0).*cprime_ge+(cprime_ge<=0)*10^-10;
     muprime_ge=cprime_ge.^(-gamma);
   

% Expectation term in the Euler equation (condition (5) in the paper)
   
    expec=(muprime_bu.*((1-delta)*ones4+irate_b)).*prob_bu+...
    (muprime_be.*((1-delta)*ones4+irate_b)).*prob_be+...
    (muprime_gu.*((1-delta)*ones4+irate_g)).*prob_gu+...
    (muprime_ge.*((1-delta)*ones4+irate_g)).*prob_ge;
   
   cn=(beta*expec).^(-1/gamma); 
   % current consumption found from the Euler equation (5) 
   % if the borrowing constraint is not binding
                    
   kprimen=wealth-cn; 
   % new capital function (i.e., condition (5) in the paper)
    
   kprimen=(kprimen>=k_min).*(kprimen<=k_max).*kprimen+(kprimen<k_min)*k_min+(kprimen>k_max)*k_max; 
   % restricting kprimen to be in [km_min,km_max] range 
   
  
   dif_k=max(max(max(max(abs(kprimen-kprime))))); 
   % difference between the new and previous capital functions
   
   kprime=update_k*kprimen+(1-update_k)*kprime; 
   % updating kprimen (condition (6) in the paper)
      
   end

% Consumption function

c=wealth-kprime; % follows from the budget constraint
